#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::NumSeq::PlanePathDelta;
use Math::PlanePath::KochCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::KochCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\node (start) at (0,0) [my dot];\n";
print "  \\node (end) at ($x_hi/2,{$y_hi*sqrt(3)/2}) [my dot];\n";
{ my ($x,$y) = $path->n_to_xy($n_hi*1/4);
  print "  \\coordinate (p1) at ($x/2,{$y*sqrt(3)/2});\n";
}
{ my ($x,$y) = $path->n_to_xy($n_hi*2/4);
  print "  \\coordinate (p2) at ($x/2,{$y*sqrt(3)/2});\n";
}
{ my ($x,$y) = $path->n_to_xy($n_hi*3/4);
  print "  \\coordinate (p3) at ($x/2,{$y*sqrt(3)/2});\n";
}

my @dir;
$Printwrap = 2;
printwrap("\\draw (start)");
$Printwrap += 2;
my ($prev_x,$prev_y) = $path->n_to_xy($n_lo);
foreach my $n ($n_lo+1 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  printwrap("--($x/2,{$y*sqrt(3)/2})");

  my $dx = $x - $prev_x;
  my $dy = $y - $prev_y;
  my $dir6 = Math::NumSeq::PlanePathDelta::_delta_func_TDir6($dx,$dy);
  push @dir, $dir6;

  ($prev_x,$prev_y) = ($x,$y);
}
printwrap("--(end);\n");

{
  $Printwrap = 0;
  printwrap_indent("% GP-Test  ");
  printwrap_suffix(" \\");
  my $len = scalar(@dir);
  printwrap("vector($len,n,n--; PmOneBits(n)%6) == [");
  $Printwrap += 2;
  foreach my $i (0 .. $#dir) {
    printwrap("$dir[$i]". ($i==$#dir ? '' : ','));
  }
  printwrap_suffix("");
  printwrap("]\n");
}

print "  % end generated\n";
exit 0;
