#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::AlternatePaper 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate;
use MyGraphs;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_k = 5;
if (@ARGV) { $max_k = shift @ARGV; }

my $Wpar_max_w = 7;
my $Wpar_max_h = 5;
my $TWpar_max_w = 9;
my $TWpar_max_h = 7;

print "% generated by tools/$FindBin::Script $max_k\n";

{
  # TW and Tdiameter

  my @TW;
  my @Tdiameter;
  foreach my $k (0 .. $max_k) {
    ### $k
    my $graph = MyAlternate::twinalternate_Graph($k);
    push @Tdiameter, scalar($graph->diameter);
    push @TW, MyGraphs::Graph_Wiener_index($graph);
  }

  foreach my $elem (['TW',\@TW],
                    ['Tdiameter',\@Tdiameter],
                   ) {
    my ($name,$aref) = @$elem;
    $Printwrap = 0;
    printwrap_indent("% GP-DEFINE  ");
    printwrap("${name}_generated_samples = {[");
    $Printwrap += 2;
    foreach my $i (0 .. $#$aref) {
      my $value = $aref->[$i];
      printwrap($value . ($i<$#$aref?',':''));
    }
    printwrap_indent("% GP-DEFINE  ");
    printwrap("]};\n");
  }
}
{
  # Wpar

  my @Wpar;
  {
    foreach my $w (1 .. $Wpar_max_w) {
      foreach my $h (1 .. $Wpar_max_h) {
        my $graph = MyAlternate::parallelogram_Graph($w,$h);
        $Wpar[$h-1][$w-1] = MyGraphs::Graph_Wiener_index($graph);
      }
    }
  }
  $Printwrap = 0;
  printwrap("Wpar_generated_samples = {[\n");
  $Printwrap += 2;
  foreach my $r (0 .. $#Wpar) {
    my $row = $Wpar[$r];
    foreach my $i (0 .. $#$row) {
      my $value = $row->[$i];
      printwrap($value . ($i<$#$row?',' : $r<$#Wpar ? ";\n" : ''));
    }
  }
  printwrap_newline();
  $Printwrap = 0;
  printwrap("]};\n");
}

{
  # TWeven and TWodd

  # delete vertex $v from $graph, the vertex must exist
  sub Graph_delete_vertex {
    my ($graph, $v) = @_;
    $graph->has_vertex($v) or die "graph has no vertex ",$v;
    $graph->delete_vertex($v);
  }

  # delete edge $u,$v from $graph, the edge must exist
  sub Graph_delete_edge {
    my ($graph, $u, $v) = @_;
    $graph->has_edge($u,$v) or die "graph has no edge $u to $v";
    $graph->delete_edge($u, $v);
  }

  sub TWpar_Graph {
    my ($w,$h) = @_;
    ### $w
    ### $h
    my $graph = MyAlternate::parallelogram_Graph($w,$h);
    $graph->set_graph_attribute(name => "TWpar $w,$h");
    my $xmax = $w-1;
    my $ymax = $h-1;
    my $tx = $xmax - $ymax;  # X of top right corner
    if ($w >= 2) {
      Graph_delete_vertex($graph, "$xmax,0");
      Graph_delete_vertex($graph, (-$ymax).",$ymax");
      $xmax--;
      for (my $x = 1; $x < $xmax; $x+=2) {
        Graph_delete_edge($graph, "$x,0",  ($x+1).",0");
        if ($h > 1) {
          Graph_delete_edge($graph, ($tx-$x).",$ymax",  ($tx-$x-1).",$ymax");
        }
      }
    }
    return $graph;
  }

  my $w_start = 2;
  my $h_start = 3;
  my @TWpar;
  {
    foreach my $w ($w_start .. $TWpar_max_w) {
      foreach my $h ($h_start .. $TWpar_max_h) {
        my $graph = TWpar_Graph($w,$h);
        $TWpar[$h-3][$w-2] = MyGraphs::Graph_Wiener_index($graph);
        # MyGraphs::Graph_view($graph);
      }
    }
  }
  $Printwrap = 0;
  print "% GP-DEFINE  \\\\ rows h>=$h_start, columns w>=$w_start\n";
  printwrap("TWpar_generated_samples = {[\n");
  $Printwrap += 2;
  foreach my $r (0 .. $#TWpar) {
    my $row = $TWpar[$r];
    foreach my $i (0 .. $#$row) {
      my $value = $row->[$i];
      printwrap($value . ($i<$#$row?',' : $r<$#TWpar ? ";\n" : ''));
    }
  }
  printwrap_newline();
  $Printwrap = 0;
  printwrap("]};\n");
}

print "% end generated\n";
exit 0;
