#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer';
use MyPrintwrap ':all';


my $start = 30;
if (@ARGV) { $start = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;

my $end = $start + 14;

print "  \\newcommand\\MyN{$start}\n";
print "  \\newcommand\\MyNturn{",$start+1,"}\n";
print "  \\newcommand\\MyEndTurn{",$end-1,"}\n";
{
  my ($x,$y) = $path->n_to_xy($start+1);
  print "  \\node (start-turn) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($end-1);
  print "  \\node (end-turn) at ($x,$y) [my dot];\n";
}

{
  $Printwrap = 2;
  printwrap("\\draw[->] ");
  $Printwrap += 2;
  my $join = '';
  foreach my $n ($start .. $end) {
    my ($x,$y) = $path->n_to_xy($n);

    my @n_list = $path->xy_to_n_list($x,$y);
    @n_list = grep {$_>=$start && $_<=$end} @n_list;
    if (@n_list >= 2) {
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("$join($sx,$sy)");
      printwrap("--($tx,$ty)");
    } else {
      printwrap("$join($x,$y)");
    }
    $join = '--';
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
